package dev.fitko.fitconnect.api.config;

import dev.fitko.fitconnect.api.exceptions.InitializationException;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.net.URI;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ApplicationConfig {

    @Builder.Default
    private String httpProxyHost = "";
    @Builder.Default
    private Integer httpProxyPort = 0;
    @Builder.Default
    private Integer requestTimeoutInSeconds = 30;

    @Builder.Default
    private URI setSchemaWriteVersion = SchemaConfig.SET_V_1_0_1.getSchemaUri();

    @Builder.Default
    private URI metadataSchemaWriteVersion = SchemaConfig.METADATA_V_1_0_0.getSchemaUri();

    private SenderConfig senderConfig;

    private SubscriberConfig subscriberConfig;

    @Builder.Default
    private Map<EnvironmentName, Environment> environments = Collections.emptyMap();

    private EnvironmentName activeEnvironment;

    private BuildInfo buildInfo;

    public Environment getEnvironmentByName(final EnvironmentName environmentName) {
        if (environments.containsKey(environmentName)) {
            return environments.get(environmentName);
        } else {
            throw new InitializationException("No environment with name '" + environmentName.getName() + "' found. Available environments are: " + getAvailableEnvironmentNames());
        }
    }

    public Environment getCurrentEnvironment() {
        return getEnvironmentByName(activeEnvironment);
    }

    public String getOAuthTokenEndpoint() {
        return getCurrentEnvironment().getAuthBaseUrl() + ResourcePaths.AUTH_TOKEN_PATH;
    }

    public String getSubmissionsEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.SUBMISSIONS_PATH;
    }

    public String getSubmissionEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.SUBMISSION_PATH;
    }

    public String getAttachmentEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.SUBMISSION_ATTACHMENT_PATH;
    }

    public String getEventsEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.EVENTS_PATH;
    }

    public String getDestinationsEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.DESTINATIONS_PATH;
    }

    public String getDestinationsKeyEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.DESTINATIONS_KEY_PATH;
    }

    public String getWellKnownKeysEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.WELL_KNOWN_KEYS_PATH;
    }

    private String getSubmissionBaseUrl() {
        return getCurrentEnvironment().getSubmissionBaseUrl();
    }

    private String getAvailableEnvironmentNames() {
        return environments.keySet()
                .stream()
                .map(EnvironmentName::getName)
                .collect(Collectors.joining(" | "));
    }
}

